import threading
import socket
import sys
import time

DEFAULT_BUFFER_SIZE = 1024

def testHandler(easySock, peerAddress, easyServer):
    print "Got request from ", peerAddress

def getHostName():
    return socket.getfqdn()

def getHostIPAddress():
    hostname, aliasList, ipAddrList = socket.gethostbyaddr(socket.getfqdn())
    for i in ipAddrList:
        if i != "127.0.0.1":
            return i
    return None


class EasySocket:
    def __init__(self, sock, handler=None, bufferSize=DEFAULT_BUFFER_SIZE):
        self.sock = sock
        self.handler = handler
        self.bufferSize = bufferSize
        self.sentSocketClosed = 0
        
        # Spin off a thread to handle incoming data
        thread = threading.Thread(target=self.listenToSocket)
        thread.start()

    def __del__(self):
        self.close()

    def listenToSocket(self):
        try:
            data = ""
            while 1:
                data = self.sock.recv(self.bufferSize)
                if data == "":
                    # The socket must have been closed!
                    self.close()
                    return
                if self.handler != None:
                    # Do we want this dispatch to happen in a different
                    # thread, so that if the handler breaks, the thread
                    # listening to the socket doesn't hang?   I've *not*
                    # done that here because it seems like having to
                    # write potentially re-entrant handler code would
                    # be more complex, and the idea that one message
                    # at a time will be pushed to the handler seems more
                    # intuitive...
                    self.handler.handleIncomingMessage(data, self)
        except socket.error:
            # Miscellaneous networking problem
            self.close()            

    def setHandler(self, handler):
        # We might want this to do throw some kind of error if you try to
        # change the handler and redirect the socket's output (?)
        self.handler = handler

    def getLocalPortNumber(self):
        return self.sock.addr[1]
        
    def send(self, data):
    	#print "sending: " + data
        self.sock.send(data)

    def close(self):
        self.sock.close()
        
        if self.sentSocketClosed == 0 and self.handler != None:
            self.sentSocketClosed = 1
            self.handler.handleSocketClosed(self)


class EasySocketHandler:
    """This is a template class for handling messages from a EasySocket.
    
    In order to receive data messages and connection close notifications
    from a EasySocket, you should have a class that extends this
    class, re-implementing the two stub methods with whatever code you
    would like executed when messages arrive or the connection is broken.
    Additionally, the handler instance should either be given as a
    parameter when the EasySocket (or EasyClient) is created,
    or using the EasySocket's setHandler() method if the EasySocket
    has been provided to your program by an EasyServer.
    
    The handleIncomingMessage method will be called whenever a packet
    of data has been received from the remote computer via the
    EasySocket.  The 'data' argument contains thedata string, and the
    'source' argument contains a reference back to the EasySocket
    that generated the message notification.

    EasySockets will only generate one call to your handler's
    handleIncomingMessage method at a time, and will simply queue up
    additional incoming characters until your handler returns.  If your
    implementation of handleIncomingMessage hangs or fails, no more
    incoming messages will be passed along to your handler.  (Be careful!)
    
    The handleSocketClosed method will be called once (and only once)
    when an EasySocket is closed, either due to the remote computer
    ending the connection, a network connectivity problem, or when the
    EasySocket's close() method has been called.  The 'source'
    argument contains a reference back to the EasySocket that has
    been closed.
    """
    
    def handleIncomingMessage(self, data, source):
        pass
        
    def handleSocketClosed(self, source):
        pass
        

class EasyClient(EasySocket):
    """This is a class that wraps basic client-side sockets.
    """

    def __init__(self, serverHost, serverPort, handler=None, bufferSize=DEFAULT_BUFFER_SIZE):
        self.serverHost = serverHost
        self.serverPort = serverPort
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect(self.serverHost, int(self.serverPort))

	time.sleep(1)

        EasySocket.__init__(self, sock, handler, bufferSize)        


class EasyServer:
    """This is a class to easily create multi-threaded servers.

    To create a new EasyServer, you pass it the port on which you
    want your server to be registered (0 means any random port), along
    with a 'handler' method, which will be called whenever a new
    client connects.

    This handler method is where the work for processing a client should
    happen.  It will be passed the following arguments:

        handler(easySock, clientAddress, source)

    The 'easySock' argument is an EasySocket connected to the client.
    The 'peerAddress' argument is the IP address of the client.
    The 'source' argument is simply a reference back to the
        EasyServer instance that dispatched the handler.

    Internally, the class generates a new thread to block on listen()
    (so that the caller doesn't block).  Additionally, each client
    request is handled by a new thread, to ensure that the server
    won't hang if a user's request handler code is broken.
    """

    def __init__(self, port, handler):
        self.port = port
        self.handler = handler
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    def __del__(self):
        self.close()

    def getLocalPortNumber(self):
        return self.sock.sock.getLocalPort()

    def serve(self):
        # keep the caller from blocking
        thread = threading.Thread(target=self.listenForClients)
        thread.start()

    def listenForClients(self):
        self.sock.bind('', self.port)
        print 'Running on port ' + str(self.port) + '...'
        self.sock.listen(5)
        while 1:
            self.handleRequest()

    def handleRequest(self):
        try:
            requestSock, peerAddress = self.sock.accept()
        except socket.error:
            return
            
        try:
            handlerWrapper = HandlerWrapper(self.handler,
                                            requestSock,
                                            peerAddress,
                                            self)
        except:
            self.handleError(requestSock, peerAddress)
            requestSock.close()

    def handleError(self, request, peerAddress):
        print '-' * 40
        print "Exception during processing of request from",
        print peerAddress
        import traceback
        traceback.print_exc()
        print '-' * 40

    def close(self):
        self.sock.close()


class HandlerWrapper:
    def __init__(self, handler, requestSock, peerAddress, server):
        self.handler = handler
        self.easySock = EasySocket(requestSock)
        self.peerAddress = peerAddress
        self.server = server

        self.thread = threading.Thread(target=self.invokeHandler)
        self.thread.start()

    def invokeHandler(self):
        closeConnection = 1
        try:
            closeConnection = self.handler(self.easySock, 
                                           self.peerAddress,
                                           self.server)
        finally:
            sys.exc_traceback = None
            
            if closeConnection:
                self.connection.close()

    
        
